@@ -15,6 +15,7 @@ Format of PROXY_LIST server entries:
 Example proxies in PROXY_LIST below are not guaranteed to be active or secure.
 If you don't already have a proxy server to connect to,
 you can try finding one from one of following sites:
+* https://www.sslproxies.org/
 * https://bit.ly/36GtZa1
 * https://www.us-proxy.org/
 * https://hidemy.name/en/proxy-list/
@@ -22,8 +23,9 @@ you can try finding one from one of following sites:
 """
 
 PROXY_LIST = {
-    "example1": "152.179.12.86:3128",  # (Example) - set your own proxy here
-    "example2": "socks4://50.197.210.138:32100",  # (Example)
+    "example1": "98.8.195.160:443",  # (Example) - set your own proxy here
+    "example2": "200.174.198.86:8888",  # (Example)
+    "example3": "socks5://184.178.172.5:15303",  # (Example)
     "proxy1": None,
     "proxy2": None,
     "proxy3": None,
