@@ -1,25 +1,46 @@
-# -*- coding: utf-8 -*-
 import codecs
 import os
 import shutil
 import sys
 import time
+from contextlib import suppress
 from seleniumbase import config as sb_config
 from seleniumbase.config import settings
 from seleniumbase.fixtures import constants
+from seleniumbase.fixtures import shared_utils
+
+python3_11_or_newer = False
+if sys.version_info >= (3, 11):
+    python3_11_or_newer = True
+py311_patch2 = constants.PatchPy311.PATCH
+
+
+def __is_cdp_swap_needed(driver):
+    """If the driver is disconnected, use a CDP method when available."""
+    return shared_utils.is_cdp_swap_needed(driver)
 
 
 def log_screenshot(test_logpath, driver, screenshot=None, get=False):
     screenshot_name = settings.SCREENSHOT_NAME
-    screenshot_path = "%s/%s" % (test_logpath, screenshot_name)
+    screenshot_path = os.path.join(test_logpath, screenshot_name)
+    screenshot_skipped = constants.Warnings.SCREENSHOT_SKIPPED
     screenshot_warning = constants.Warnings.SCREENSHOT_UNDEFINED
+    if (
+        (hasattr(sb_config, "no_screenshot") and sb_config.no_screenshot)
+        or screenshot == screenshot_skipped
+    ):
+        if get:
+            return screenshot
+        return
     try:
         if not screenshot:
-            element = driver.find_element_by_tag_name("body")
+            element = driver.find_element("tag name", "body")
             screenshot = element.screenshot_as_base64
         if screenshot != screenshot_warning:
             with open(screenshot_path, "wb") as file:
                 file.write(screenshot)
+            with suppress(Exception):
+                shared_utils.make_writable(screenshot_path)
         else:
             print("WARNING: %s" % screenshot_warning)
         if get:
@@ -38,6 +59,7 @@ def get_master_time():
     timestamp = str(int(time.time())) + "  (Unix Timestamp)"
     now = datetime.datetime.now()
     utc_offset = -time.timezone / 3600.0
+    utc_offset += time.daylight
     utc_str = "UTC+0"
     if utc_offset > 0:
         if utc_offset < 10:
@@ -67,6 +89,12 @@ def get_master_time():
 
 
 def get_browser_version(driver):
+    if (
+        python3_11_or_newer
+        and py311_patch2
+        and hasattr(sb_config, "_browser_version")
+    ):
+        return sb_config._browser_version
     driver_capabilities = driver.capabilities
     if "version" in driver_capabilities:
         browser_version = driver_capabilities["version"]
@@ -76,15 +104,14 @@ def get_browser_version(driver):
 
 
 def get_driver_name_and_version(driver, browser):
+    if hasattr(sb_config, "_driver_name_version"):
+        return sb_config._driver_name_version
     if driver.capabilities["browserName"].lower() == "chrome":
         cap_dict = driver.capabilities["chrome"]
         return ("chromedriver", cap_dict["chromedriverVersion"].split(" ")[0])
     elif driver.capabilities["browserName"].lower() == "msedge":
         cap_dict = driver.capabilities["msedge"]
         return ("msedgedriver", cap_dict["msedgedriverVersion"].split(" ")[0])
-    elif driver.capabilities["browserName"].lower() == "opera":
-        cap_dict = driver.capabilities["opera"]
-        return ("operadriver", cap_dict["operadriverVersion"].split(" ")[0])
     elif driver.capabilities["browserName"].lower() == "firefox":
         return ("geckodriver", driver.capabilities["moz:geckodriverVersion"])
     elif browser == "safari":
@@ -103,6 +130,8 @@ def log_test_failure_data(test, test_logpath, driver, browser, url=None):
     browser_version = None
     driver_version = None
     driver_name = None
+    duration = None
+    exc_message = None
     try:
         browser_version = get_browser_version(driver)
     except Exception:
@@ -113,20 +142,29 @@ def log_test_failure_data(test, test_logpath, driver, browser, url=None):
         )
     except Exception:
         pass
+    try:
+        duration = "%.2fs" % (time.time() - (sb_config.start_time_ms / 1000.0))
+    except Exception:
+        duration = "(Unknown Duration)"
     if browser_version:
         headless = ""
         if test.headless and browser in ["chrome", "edge", "firefox"]:
             headless = " / headless"
-        browser_displayed = "%s (%s%s)" % (browser, browser_version, headless)
+        if test.headless2 and browser in ["chrome", "edge"]:
+            headless = " / headless2"
+        if browser and len(browser) > 1:
+            # Capitalize the first letter
+            browser = "%s%s" % (browser[0].upper(), browser[1:])
+        browser_displayed = "%s %s%s" % (browser, browser_version, headless)
         if driver_name and driver_version:
-            driver_displayed = "%s (%s)" % (driver_name, driver_version)
-    if not browser_version:
+            driver_displayed = "%s %s" % (driver_name, driver_version)
+    else:
         browser_displayed = browser
         driver_displayed = "(Unknown Driver)"
     if not driver_displayed:
         driver_displayed = "(Unknown Driver)"
     basic_info_name = settings.BASIC_INFO_NAME
-    basic_file_path = "%s/%s" % (test_logpath, basic_info_name)
+    basic_file_path = os.path.join(test_logpath, basic_info_name)
     if url:
         last_page = url
     else:
@@ -140,6 +178,7 @@ def log_test_failure_data(test, test_logpath, driver, browser, url=None):
         "--------------------------------------------------------------------"
     )
     data_to_save.append("Last Page: %s" % last_page)
+    data_to_save.append(" Duration: %s" % duration)
     data_to_save.append("  Browser: %s" % browser_displayed)
     data_to_save.append("   Driver: %s" % driver_displayed)
     data_to_save.append("Timestamp: %s" % timestamp)
@@ -149,9 +188,9 @@ def log_test_failure_data(test, test_logpath, driver, browser, url=None):
         "--------------------------------------------------------------------"
     )
     if (
-        sys.version_info[0] >= 3
-        and hasattr(test, "_outcome")
-        and (hasattr(test._outcome, "errors") and test._outcome.errors)
+        hasattr(test, "_outcome")
+        and hasattr(test._outcome, "errors")
+        and test._outcome.errors
     ):
         try:
             exc_message = test._outcome.errors[0][1][1]
@@ -159,30 +198,58 @@ def log_test_failure_data(test, test_logpath, driver, browser, url=None):
             traceback_list = traceback.format_list(
                 traceback.extract_tb(traceback_address)[1:]
             )
+            updated_list = []
+            counter = 0
+            for traceback_item in traceback_list:
+                if "self._callTestMethod(testMethod)" in traceback_item:
+                    counter = 1
+                    updated_list.append(traceback_item)  # In case not cleared
+                    continue
+                elif (
+                    ", in _callTestMethod" in traceback_item.strip()
+                    and "method()" in traceback_item.strip()
+                    and counter == 1
+                ):
+                    counter = 0
+                    updated_list = []
+                    continue
+                else:
+                    counter = 0
+                    updated_list.append(traceback_item)
+            traceback_list = updated_list
             traceback_message = "".join(traceback_list).strip()
         except Exception:
             exc_message = "(Unknown Exception)"
             traceback_message = "(Unknown Traceback)"
-        data_to_save.append("Traceback: " + traceback_message)
-        data_to_save.append("Exception: " + str(exc_message))
+        traceback_message = str(traceback_message).strip()
+        data_to_save.append("Traceback:\n  %s" % traceback_message)
+        data_to_save.append("Exception: %s" % exc_message)
     else:
-        the_traceback = None
+        traceback_message = None
         if hasattr(test, "is_behave") and test.is_behave:
             if sb_config.behave_scenario.status.name == "failed":
-                if sb_config.behave_step.error_message:
-                    the_traceback = sb_config.behave_step.error_message
+                if (
+                    hasattr(sb_config, "behave_step")
+                    and hasattr(sb_config.behave_step, "error_message")
+                    and sb_config.behave_step.error_message
+                ):
+                    traceback_message = sb_config.behave_step.error_message
         else:
-            the_traceback = "".join(
-                traceback.format_exception(
-                    sys.exc_info()[0],
-                    sys.exc_info()[1],
-                    sys.exc_info()[2],
-                )
+            format_exception = traceback.format_exception(
+                sys.exc_info()[0], sys.exc_info()[1], sys.exc_info()[2]
             )
+            if format_exception:
+                updated_list = []
+                for line in format_exception:
+                    if "sb_manager.py" in line and "yield sb" in line:
+                        continue
+                    updated_list.append(line)
+                format_exception = updated_list
+            traceback_message = "".join(format_exception)
         if (
-            not the_traceback
-            or len(str(the_traceback)) < 30
-            or the_traceback.endswith("StopIteration\n")
+            not traceback_message
+            or len(str(traceback_message)) < 30
+            or traceback_message.endswith("StopIteration\n")
         ):
             good_stack = []
             the_stacks = []
@@ -190,6 +257,10 @@ def log_test_failure_data(test, test_logpath, driver, browser, url=None):
                 the_stacks = traceback.format_list(
                     traceback.extract_tb(sys.last_traceback)
                 )
+            elif hasattr(sb_config, "_excinfo_tb"):
+                the_stacks = traceback.format_list(
+                    traceback.extract_tb(sb_config._excinfo_tb)
+                )
             else:
                 message = None
                 if hasattr(test, "is_behave") and test.is_behave:
@@ -201,17 +272,36 @@ def log_test_failure_data(test, test_logpath, driver, browser, url=None):
                 if "/site-packages/pluggy/" not in stack:
                     if "/site-packages/_pytest/" not in stack:
                         good_stack.append(stack)
-            the_traceback = "".join(good_stack)
-            data_to_save.append("Traceback: " + the_traceback)
+            traceback_message = str("".join(good_stack)).strip()
+            data_to_save.append("Traceback:\n  %s" % traceback_message)
             if hasattr(sys, "last_value"):
                 last_value = sys.last_value
                 if last_value:
-                    data_to_save.append("Exception: " + str(last_value))
+                    data_to_save.append("Exception: %s" % last_value)
+            elif hasattr(sb_config, "_excinfo_value"):
+                data_to_save.append("Exception: %s" % sb_config._excinfo_value)
         else:
-            data_to_save.append("Traceback: " + the_traceback)
-    log_file = codecs.open(basic_file_path, "w+", "utf-8")
-    log_file.writelines("\r\n".join(data_to_save))
-    log_file.close()
+            data_to_save.append("Traceback:\n  %s" % traceback_message)
+    if hasattr(test, "is_nosetest") and test.is_nosetest:
+        # Also save the data for the report
+        sb_config._report_test_id = test_id
+        sb_config._report_fail_page = last_page
+        sb_config._report_duration = duration
+        sb_config._report_browser = browser_displayed
+        sb_config._report_driver = driver_displayed
+        sb_config._report_timestamp = timestamp
+        sb_config._report_date = the_date
+        sb_config._report_time = the_time
+        sb_config._report_traceback = traceback_message
+        sb_config._report_exception = exc_message
+    if not os.path.exists(test_logpath):
+        with suppress(Exception):
+            os.makedirs(test_logpath)
+    with suppress(Exception):
+        log_file = codecs.open(basic_file_path, "w+", encoding="utf-8")
+        log_file.writelines("\r\n".join(data_to_save))
+        log_file.close()
+        shared_utils.make_writable(basic_file_path)
 
 
 def log_skipped_test_data(test, test_logpath, driver, browser, reason):
@@ -220,24 +310,25 @@ def log_skipped_test_data(test, test_logpath, driver, browser, reason):
     browser_version = None
     driver_version = None
     driver_name = None
-    try:
+    with suppress(Exception):
         browser_version = get_browser_version(driver)
-    except Exception:
-        pass
-    try:
+    with suppress(Exception):
         driver_name, driver_version = get_driver_name_and_version(
             driver, browser
         )
-    except Exception:
-        pass
     if browser_version:
         headless = ""
         if test.headless and browser in ["chrome", "edge", "firefox"]:
             headless = " / headless"
-        browser_displayed = "%s (%s%s)" % (browser, browser_version, headless)
+        if test.headless2 and browser in ["chrome", "edge"]:
+            headless = " / headless2"
+        if browser and len(browser) > 1:
+            # Capitalize the first letter
+            browser = "%s%s" % (browser[0].upper(), browser[1:])
+        browser_displayed = "%s %s%s" % (browser, browser_version, headless)
         if driver_name and driver_version:
-            driver_displayed = "%s (%s)" % (driver_name, driver_version)
-    if not browser_version:
+            driver_displayed = "%s %s" % (driver_name, driver_version)
+    else:
         browser_displayed = browser
         driver_displayed = "(Unknown Driver)"
     if not driver_displayed:
@@ -261,9 +352,11 @@ def log_skipped_test_data(test, test_logpath, driver, browser, reason):
     data_to_save.append(" * Skip Reason: %s" % reason)
     data_to_save.append("")
     file_path = os.path.join(test_logpath, "skip_reason.txt")
-    log_file = codecs.open(file_path, "w+", encoding="utf-8")
-    log_file.writelines("\r\n".join(data_to_save))
-    log_file.close()
+    with suppress(Exception):
+        log_file = codecs.open(file_path, "w+", encoding="utf-8")
+        log_file.writelines("\r\n".join(data_to_save))
+        log_file.close()
+        shared_utils.make_writable(file_path)
 
 
 def log_page_source(test_logpath, driver, source=None):
@@ -272,7 +365,11 @@ def log_page_source(test_logpath, driver, source=None):
         page_source = source
     else:
         try:
-            page_source = driver.page_source
+            page_source = None
+            if __is_cdp_swap_needed(driver):
+                page_source = driver.cdp.get_page_source()
+            else:
+                page_source = driver.page_source
             page_source = get_html_source_with_base_href(driver, page_source)
         except Exception:
             source = constants.Warnings.PAGE_SOURCE_UNDEFINED
@@ -286,10 +383,15 @@ def log_page_source(test_logpath, driver, source=None):
                 "unresponsive, or closed prematurely!</h4>"
             )
         )
-    html_file_path = "%s/%s" % (test_logpath, html_file_name)
-    html_file = codecs.open(html_file_path, "w+", "utf-8")
-    html_file.write(page_source)
-    html_file.close()
+    if not os.path.exists(test_logpath):
+        with suppress(Exception):
+            os.makedirs(test_logpath)
+    html_file_path = os.path.join(test_logpath, html_file_name)
+    with suppress(Exception):
+        html_file = codecs.open(html_file_path, "w+", encoding="utf-8")
+        html_file.write(page_source)
+        html_file.close()
+        shared_utils.make_writable(html_file_path)
 
 
 def get_test_id(test):
@@ -301,6 +403,24 @@ def get_test_id(test):
             scenario_name = scenario_name.split(" -- @")[0]
         test_id = "%s:%s => %s" % (file_name, line_num, scenario_name)
         return test_id
+    elif hasattr(test, "is_context_manager") and test.is_context_manager:
+        filename = test.__class__.__module__.split(".")[-1] + ".py"
+        classname = test.__class__.__name__
+        methodname = test._testMethodName
+        context_id = None
+        if filename == "base_case.py" or methodname == "runTest":
+            import traceback
+
+            stack_base = traceback.format_stack()[0].split(", in ")[0]
+            test_base = stack_base.split(", in ")[0].split(os.sep)[-1]
+            if hasattr(test, "cm_filename") and test.cm_filename:
+                filename = test.cm_filename
+            else:
+                filename = test_base.split('"')[0]
+            classname = "SB"
+            methodname = ".py:" + test_base.split(", line ")[-1]
+            context_id = filename.split(".")[0] + methodname + ":" + classname
+            return context_id
     test_id = None
     try:
         test_id = get_test_name(test)
@@ -316,7 +436,13 @@ def get_test_id(test):
 
 
 def get_test_name(test):
-    if test.is_pytest:
+    if "PYTEST_CURRENT_TEST" in os.environ:
+        full_name = os.environ["PYTEST_CURRENT_TEST"]
+        if "] " in full_name:
+            test_name = full_name.split("] ")[0] + "]"
+        else:
+            test_name = full_name.split(" ")[0]
+    elif test.is_pytest:
         test_name = "%s.py::%s::%s" % (
             test.__class__.__module__.split(".")[-1],
             test.__class__.__name__,
@@ -330,22 +456,16 @@ def get_test_name(test):
         )
     if test._sb_test_identifier and len(str(test._sb_test_identifier)) > 6:
         test_name = test._sb_test_identifier
-        if hasattr(test, "_using_sb_fixture_class"):
-            if test_name.count(".") >= 2:
-                parts = test_name.split(".")
-                full = parts[-3] + ".py::" + parts[-2] + "::" + parts[-1]
-                test_name = full
-        elif hasattr(test, "_using_sb_fixture_no_class"):
-            if test_name.count(".") >= 1:
-                parts = test_name.split(".")
-                full = parts[-2] + ".py::" + parts[-1]
-                test_name = full
     return test_name
 
 
 def get_last_page(driver):
     try:
-        last_page = driver.current_url
+        last_page = None
+        if __is_cdp_swap_needed(driver):
+            last_page = driver.cdp.get_current_url()
+        else:
+            last_page = driver.current_url
     except Exception:
         last_page = "[WARNING! Browser Not Open!]"
     if len(last_page) < 5:
@@ -413,8 +533,8 @@ def archive_logs_if_set(log_path, archive_logs=False):
     else:
         if settings.ARCHIVE_EXISTING_LOGS or archive_logs:
             if len(os.listdir(log_path)) > 0:
-                archived_folder = "%s/../archived_logs/" % log_path
-                archived_folder = os.path.realpath(archived_folder) + "/"
+                saved_folder = "%s/../%s/" % (log_path, constants.Logs.SAVED)
+                archived_folder = os.path.realpath(saved_folder) + "/"
                 log_path = os.path.realpath(log_path) + "/"
                 if not os.path.exists(archived_folder):
                     try:
@@ -427,22 +547,30 @@ def archive_logs_if_set(log_path, archive_logs=False):
 
 
 def log_folder_setup(log_path, archive_logs=False):
-    """Handle Logging"""
+    """Clean up logs to prepare for another run"""
     if log_path.endswith("/"):
         log_path = log_path[:-1]
+    if log_path.startswith("/"):
+        log_path = log_path[1:]
+    if constants.Logs.SAVED.endswith("/"):
+        constants.Logs.SAVED = constants.Logs.SAVED[:-1]
+    if constants.Logs.SAVED.startswith("/"):
+        constants.Logs.SAVED = constants.Logs.SAVED[1:]
+    if len(log_path) < 10 or len(constants.Logs.SAVED) < 10:
+        return  # Prevent accidental deletions if constants are renamed
     if not os.path.exists(log_path):
         try:
             os.makedirs(log_path)
         except Exception:
-            pass  # Should only be reachable during multi-threaded runs
+            pass  # Only reachable during multi-threaded runs
     else:
-        archived_folder = "%s/../archived_logs/" % log_path
-        archived_folder = os.path.realpath(archived_folder) + "/"
+        saved_folder = "%s/../%s/" % (log_path, constants.Logs.SAVED)
+        archived_folder = os.path.realpath(saved_folder) + "/"
         if not os.path.exists(archived_folder):
             try:
                 os.makedirs(archived_folder)
             except Exception:
-                pass  # Should only be reachable during multi-threaded runs
+                pass  # Only reachable during multi-threaded runs
         archived_logs = "%slogs_%s" % (archived_folder, int(time.time()))
         if len(os.listdir(log_path)) > 0:
             try:
@@ -459,3 +587,18 @@ def log_folder_setup(log_path, archive_logs=False):
                     pass
                 else:
                     shutil.rmtree(archived_logs)  # (Archive test run later)
+
+
+def clear_empty_logs():
+    latest_logs_dir = os.path.join(os.getcwd(), constants.Logs.LATEST) + os.sep
+    archived_folder = os.path.join(os.getcwd(), constants.Logs.SAVED) + os.sep
+    if os.path.exists(latest_logs_dir) and not os.listdir(latest_logs_dir):
+        try:
+            os.rmdir(latest_logs_dir)
+        except OSError:
+            pass
+    if os.path.exists(archived_folder) and not os.listdir(archived_folder):
+        try:
+            os.rmdir(archived_folder)
+        except OSError:
+            pass
