@@ -7,6 +7,7 @@ import sys
 import time
 from contextlib import suppress
 from seleniumbase import config as sb_config
+from seleniumbase.config import settings
 from seleniumbase.fixtures import constants
 
 
@@ -29,8 +30,48 @@ def pip_install(package, version=None):
             )
 
 
+def get_mfa_code(totp_key=None):
+    """Returns a time-based one-time password based on the
+    Google Authenticator algorithm for multi-factor authentication.
+    If the "totp_key" is not specified, this method defaults
+    to using the one provided in [seleniumbase/config/settings.py].
+    Google Authenticator codes expire & change at 30-sec intervals.
+    If the fetched password expires in the next 1.2 seconds, waits
+    for a new one before returning it (may take up to 1.2 seconds).
+    See https://pyotp.readthedocs.io/en/latest/ for details."""
+    import pyotp
+
+    if not totp_key:
+        totp_key = settings.TOTP_KEY
+    epoch_interval = time.time() / 30.0
+    cycle_lifespan = float(epoch_interval) - int(epoch_interval)
+    if float(cycle_lifespan) > 0.96:
+        # Password expires in the next 1.2 seconds. Wait for a new one.
+        for i in range(30):
+            time.sleep(0.04)
+            epoch_interval = time.time() / 30.0
+            cycle_lifespan = float(epoch_interval) - int(epoch_interval)
+            if not float(cycle_lifespan) > 0.96:
+                # The new password cycle has begun
+                break
+    totp = pyotp.TOTP(totp_key)
+    return str(totp.now())
+
+
+def is_arm_linux():
+    """Returns True if machine is ARM Linux.
+    This will be useful once Google adds
+    support for ARM Linux ChromeDriver.
+    (Raspberry Pi uses ARM architecture.)"""
+    return (
+        platform.system() == "Linux"
+        and platform.machine() == "aarch64"
+    )
+
+
 def is_arm_mac():
-    """(M1 / M2 Macs use the ARM processor)"""
+    """Returns True if machine is ARM Mac.
+    (Eg. M1 / M2 Macs use ARM processors)"""
     return (
         "darwin" in sys.platform
         and (
